import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/bloc/create_space_model_event.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/bloc/create_space_model_state.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/create_space_template_body_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/tag_model.dart';
import 'package:syncrow_web/services/space_model_mang_api.dart';

class CreateSpaceModelBloc
    extends Bloc<CreateSpaceModelEvent, CreateSpaceModelState> {
  SpaceTemplateModel? _space;

  final SpaceModelManagementApi _api;

  CreateSpaceModelBloc(this._api) : super(CreateSpaceModelInitial()) {
    on<CreateSpaceTemplate>((event, emit) async {
      try {
        late SpaceTemplateModel spaceTemplate = event.spaceTemplate;

        final tagBodyModels =
            spaceTemplate.tags?.map((tag) => tag.toTagBodyModel()).toList() ??
                [];

        final subspaceTemplateBodyModels =
            spaceTemplate.subspaceModels?.map((subspaceModel) {
                  final tagsubspaceBodyModels = subspaceModel.tags
                          ?.map((tag) => tag.toTagBodyModel())
                          .toList() ??
                      [];
                  return CreateSubspaceTemplateModel()
                    ..subspaceName = subspaceModel.subspaceName
                    ..tags = tagsubspaceBodyModels;
                }).toList() ??
                [];
        final spaceModelBody = CreateSpaceTemplateBodyModel(
            modelName: spaceTemplate.modelName,
            tags: tagBodyModels,
            subspaceModels: subspaceTemplateBodyModels);

        final newSpaceTemplate = await _api.createSpaceModel(spaceModelBody);
        spaceTemplate.uuid = newSpaceTemplate?.uuid ?? '';

        if (newSpaceTemplate != null) {
          emit(CreateSpaceModelLoaded(spaceTemplate));

          if (event.onCreate != null) {
            event.onCreate!(spaceTemplate);
          }
        }
      } catch (e) {
        emit(CreateSpaceModelError('Error creating space model'));
      }
    });

    on<LoadSpaceTemplate>((event, emit) {
      emit(CreateSpaceModelLoading());
      Future.delayed(const Duration(seconds: 1), () {
        if (_space != null) {
          emit(CreateSpaceModelLoaded(_space!));
        } else {
          emit(CreateSpaceModelError("No space template found"));
        }
      });
    });

    on<UpdateSpaceTemplate>((event, emit) {
      _space = event.spaceTemplate;
      emit(CreateSpaceModelLoaded(_space!));
    });

    on<AddSubspacesToSpaceTemplate>((event, emit) {
      final currentState = state;

      if (currentState is CreateSpaceModelLoaded) {
        final updatedSpace = currentState.space.copyWith(
          subspaceModels: [
            ...(_space!.subspaceModels ?? []),
            ...event.subspaces,
          ],
        );
        emit(CreateSpaceModelLoaded(updatedSpace));
      } else {
        emit(CreateSpaceModelError("Space template not initialized"));
      }
    });

    on<UpdateSpaceTemplateName>((event, emit) {
      final currentState = state;
      if (currentState is CreateSpaceModelLoaded) {
        if (event.name.trim().isEmpty) {

          emit(CreateSpaceModelLoaded(
            currentState.space,
            errorMessage: "Model name cannot be empty",
          ));
        } else {
          final updatedSpaceModel =
              currentState.space.copyWith(modelName: event.name);
         
          emit(CreateSpaceModelLoaded(updatedSpaceModel));
        }
      } else {
        emit(CreateSpaceModelError("Space template not initialized"));
      }
    });
  }
}
