import 'package:flutter/material.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/product_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/subspace_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/tag_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/button_content_widget.dart';
import 'package:syncrow_web/pages/spaces_management/create_subspace_model/views/create_subspace_model_dialog.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class SubspaceModelCreate extends StatelessWidget {
  final List<SubspaceTemplateModel> subspaces;
  final TextEditingController spaceNameController;
  final List<TagModel>? spaceTagModels;
  final List<String>? allTags;
  final List<ProductModel>? products;
  final SpaceTemplateModel? spaceModel;

  const SubspaceModelCreate(
    BuildContext context, {
    Key? key,
    required this.subspaces,
    this.spaceTagModels,
    required this.allTags,
    required this.products,
    required this.spaceModel,
    required this.spaceNameController,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final screenWidth = MediaQuery.of(context).size.width;
    return Container(
      child: subspaces.isEmpty
          ? TextButton(
              style: TextButton.styleFrom(
                overlayColor: ColorsManager.transparentColor,
              ),
              onPressed: () async {
                Navigator.of(context).pop();

                await showDialog<List<SubspaceTemplateModel>>(
                  barrierDismissible: false,
                  context: context,
                  builder: (BuildContext context) {
                    return CreateSubSpaceModelDialog(
                      allTags: allTags,
                      spaceName: spaceNameController.text,
                      spaceModel: spaceModel,
                      spaceTagModels: spaceTagModels,
                      products: products,
                      isEdit: true,
                      dialogTitle: subspaces.isEmpty
                          ? 'Create Sub-space'
                          : 'Edit Sub-space',
                      existingSubSpaces: subspaces,
                    );
                  },
                );
              },
              child: const ButtonContentWidget(
                icon: Icons.add,
                label: 'Create Sub Space',
              ),
            )
          : SizedBox(
              width: screenWidth * 0.25,
              child: Container(
                padding: const EdgeInsets.all(8.0),
                decoration: BoxDecoration(
                  color: ColorsManager.textFieldGreyColor,
                  borderRadius: BorderRadius.circular(15),
                  border: Border.all(
                    color: ColorsManager.textFieldGreyColor,
                    width: 3.0, // Border width
                  ),
                ),
                child: Wrap(
                  spacing: 8.0,
                  runSpacing: 8.0,
                  children: [
                    ...subspaces.map(
                      (subspace) => Chip(
                        label: Text(
                          subspace.subspaceName,
                          style: const TextStyle(
                              color: ColorsManager.spaceColor), // Text color
                        ),
                        backgroundColor: ColorsManager.whiteColors, // Chip background color
                        shape: RoundedRectangleBorder(
                          borderRadius:
                              BorderRadius.circular(16), // Rounded chip
                          side: const BorderSide(
                              color: ColorsManager.spaceColor), // Border color
                        ),
                      ),
                    ),
                    GestureDetector(
                      onTap: () async {
                        Navigator.of(context).pop();
                        await showDialog<List<SubspaceTemplateModel>>(
                          barrierDismissible: false,
                          context: context,
                          builder: (BuildContext context) {
                            return CreateSubSpaceModelDialog(
                              isEdit: true,
                              dialogTitle: 'Edit Sub-space',
                              existingSubSpaces: subspaces,
                              allTags: allTags,
                              spaceName: spaceNameController.text,
                              spaceTagModels: spaceTagModels,
                              products: products,
                              spaceModel: spaceModel,
                            );
                          },
                        );
                      },
                      child: Chip(
                        label: const Text(
                          'Edit',
                          style: TextStyle(
                              color: ColorsManager.spaceColor), 
                        ),
                        backgroundColor:
                            ColorsManager.whiteColors,
                        shape: RoundedRectangleBorder(
                          borderRadius:
                              BorderRadius.circular(16), 
                          side: const BorderSide(
                              color: ColorsManager.spaceColor), 
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
    );
  }
}
